// For Linux icpc
//20180126	self-compatible hermaphrodite

#include <boost/random.hpp>
#include <random>
#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <time.h>
#include <vector>
#include <numeric>
#include <algorithm>
using namespace std;

int main(int argc,char *argv[])
{
	int i, j;
	cout <<  "argv 1: " << argv[1] << endl;
	//Read parameters from txt file
	double Parameters[10] = {};
	ifstream inputfile("Parameters.txt", ios::in); //read only
	string parameter_name;
	if (inputfile.fail()){
		cout << "Cannot find parameter file(Parameters.txt). press Enter key." << endl;
		getchar();
		return -1;
	}
	i = 0;
	while (true){//
		if (i % 2 == 0){
			inputfile >> parameter_name;
			if (inputfile.fail()) break;
			cout << parameter_name << " = ";
		}
		else {
			inputfile >> Parameters[i / 2];
			if (inputfile.fail()) break;
			cout << Parameters[i / 2] << endl;
		}
		i++;
	}
	inputfile.close();
	cout << "parameter read end" << endl;

/*The example of "Parameters.txt"
WholePopulationSize			1000000
PopulationSizeForSeed		1000000
PopulationSizeForPollen		100
Num_species					10
MutationRate				0.01
DeathRate					1.0
DispersalRate				1.0
TimeSteps					10000
*/

	//Setting
	int WholePopulationSize = int(Parameters[0]);
	int PopulationSizeForSeed = int(Parameters[1]);
	int PopulationSizeForPollen = int(Parameters[2]);
	int Num_species = int(Parameters[3]) + 1;	//add for the vacant site
	double MutationRate = Parameters[4];
	double DeathRate = Parameters[5];
	double DispersalRate = Parameters[6];
	int TimeSteps = int(Parameters[7]);
	double Minimum = numeric_limits<double>::min();		//minimum value in positive double

	/*		//to assess single parameter by argv[1]. This is used for Fig.1
	try {
		int num = stoi(argv[1]);
		PopulationSizeForPollen = num;
		WholePopulationSize = WholePopulationSize * num;
		PopulationSizeForSeed = PopulationSizeForSeed * num;
		cout << "PopulationSizeForPollen replaced by " << num << endl;
		cout << "WholePopulationSize replaced by " << WholePopulationSize << endl;
		cout << "PopulationSizeForSeed replaced by " << PopulationSizeForSeed << endl;
	}
		catch (const invalid_argument& e) {
		cout << "argv[1]: invalid argument" << endl;
	}
		catch (const out_of_range& e) {
		cout << "argv[1]: out of range" << endl;
	}
	//*/
	
	if (WholePopulationSize % PopulationSizeForPollen != 0){
		cout << "WholePopulationSize:" << WholePopulationSize << endl;
		cout << "PopulationSizeForPollen:" << PopulationSizeForPollen << endl;
		cout << "WholePopulationSize is indivisible by PopulationSizeForPollen. press any key." << endl;
		getchar();
		return -1;
	}
	if (WholePopulationSize % PopulationSizeForSeed != 0){
		cout << "WholePopulationSize:" << WholePopulationSize << endl;
		cout << "PopulationSizeForSeed:" << PopulationSizeForSeed << endl;
		cout << "WholePopulationSize is indivisible by PopulationSizeForSeed. press any key." << endl;
		getchar();
		return -1;
	}
	int Num_MatingPopulation = WholePopulationSize / PopulationSizeForPollen;

	//allocate memory
	vector<unsigned short> Species(WholePopulationSize, 0);										//Species
	vector<double> Harassment(WholePopulationSize, Minimum);									//amount of Harassment (0-1: reduction rate of productiviy)
	vector<unsigned short> N_Species(WholePopulationSize, 0);									//Species for Next population
	vector<double> N_Harassment(WholePopulationSize, 0.0);										//amount of Harassment for Next population
	vector<vector<double>> THarassment(Num_MatingPopulation, vector<double>(Num_species, 0.0));	//total Harassment of each species at each subpopulation
	vector<double> SeedCounter(WholePopulationSize, 0);											//amount of produced seeds

	//allocate memory for output at each end of timestep
	vector<vector<double>> Harassment_sp(TimeSteps, vector<double>(Num_species, 0.0));				//Total amount of Harassment of each species
	vector<vector<int>> Number_sp(TimeSteps, vector<int>(Num_species, 0));							//Total number of individuals of each species
	vector<vector<double>> RealizedProductivity_sp(TimeSteps, vector<double>(Num_species,0.0));		//Realized amount of seeds of each species

	//Random function
	random_device rd;   // non-deterministic generator
	mt19937 gen(rd());  // to seed mersenne twister.
	uniform_int_distribution<int> sp(0, (Num_species - 1));				//randomly decide species
	uniform_int_distribution<int> rbinom(0, 1);							//random binomial 
	uniform_real_distribution<double> runif(0.0, 1.0);					//random uniform
	normal_distribution<double> mutant(0.0, 0.001);						//random normal mean 0, sigma 0.001, for mutation
	normal_distribution<double> randomProduct(2, 0.1);					//random fecundity for each species


	vector<double> Productivity(Num_species, 0);			//relative fecundity for each species (from 2 to 1)
	cout << "Productivity: ";
	int n = (Num_species - 1);
	double dif = 1/double(n);
	for (i = 0; i < n; ++i){
		Productivity[i] = 1.0 + dif * (i + 1);
		//Productivity[i] = randomProduct(gen);		//randomly decide species specific fecundity
		if(n == 1){Productivity[i] = 10;}		
		cout << Productivity[i] << ", ";
	}
	cout << Productivity[n] << ", " << endl;	//This is productivity of vacant sites (0)

	//cout << "-------------------------------Set Time--------------------------------" << endl;
	time_t tt = time(NULL);
	const tm* lt = localtime(&tt);
	char time_str[81];
	strftime(time_str,sizeof(time_str),"%Y%m%d_%H%I%S", lt);
	cout << "Simulation start at " << time_str << endl;


	//initialize population	
	double p = double(PopulationSizeForPollen);
	for (i = 0; i < WholePopulationSize; ++i){
		Species[i] = sp(gen);
		Harassment[i] = Minimum;			//To avoid extinction at first few generations
	}


	for (int t = 0; t < TimeSteps; ++t) {
		//cout << "-----------------------------Timestep start----------------------------" << endl;
		//cout << "Generation:" << t << endl;


		//cout << "--------------------------------Harassment-------------------------------" << endl;
		for (int x = 0; x < Num_MatingPopulation; ++x){		//reset
			for (i = 0; i < Num_species; ++i){
				THarassment[x][i] = 0.0;
			}
		}
		for (i = 0; i < WholePopulationSize; ++i){
			int subpop = i / PopulationSizeForPollen;
			THarassment[subpop][Species[i]] += Harassment[i];		//total amount of Harassment for each species at the i-th local population
		}

		//cout << "-----------------------------Seed production-----------------------------" << endl;
		double RealizedProductivity = Productivity[Species[0]] * (1.0 - THarassment[0][Species[0]]);
		//double RealizedProductivity = Productivity[Species[0]] * (1.0 - THarassment[0][Species[0]] + Harassment[0]);	//self incompatible
		//if(RealizedProductivity > 0.0 && THarassment[0][Species[0]] > Harassment[0]){		//self incompatible = others must exist at the local population
		if(RealizedProductivity > 0.0){
			SeedCounter[0] = RealizedProductivity;
			RealizedProductivity_sp[t][Species[0]] += RealizedProductivity;
		}else{
			//cout << "fail to reproduction at 0 and " << THarassment[0][Species[0]] << endl;
			SeedCounter[0] = 0.0;
		}
		for (i = 1; i < WholePopulationSize; ++i){
			int subpop = i / PopulationSizeForPollen;
			RealizedProductivity = Productivity[Species[i]] * (1.0 - THarassment[subpop][Species[i]]);
			//RealizedProductivity = Productivity[Species[i]] * (1.0 - THarassment[subpop][Species[i]] + Harassment[i]);	//self incompatible
			//if(RealizedProductivity > 0.0 && THarassment[subpop][Species[i]] > Harassment[i]){	//self incompatible = others must exist at the local population
			if(RealizedProductivity > 0.0){
				SeedCounter[i] = RealizedProductivity + SeedCounter[i - 1];
				RealizedProductivity_sp[t][Species[i]] += RealizedProductivity;	//for output
			}else{
				SeedCounter[i] = SeedCounter[i - 1];	//fail to reproduction because large amount of Harassment from mates or no mating partner
			}
		}
		double ProbVacantSite = SeedCounter[WholePopulationSize-1] / WholePopulationSize;	//Probability of vacant sites

		//cout << "--------------------------Decide Next individuals-------------------------" << endl;
		for (i = 0; i < WholePopulationSize; ++i){
			//decide the mother of seed for each site (i)
			if (DeathRate > runif(gen)){				//the site occupied by the new individual
				if(ProbVacantSite > runif(gen)){		//whether enough amount of seeds are produced
					if (DispersalRate > runif(gen)){	//the site occupied by the seed randomly chosen from the whole population
						double SeedThreshold = runif(gen) * SeedCounter[WholePopulationSize - 1];
						j = lower_bound(SeedCounter.begin(), SeedCounter.end(), SeedThreshold) - SeedCounter.begin();
					} else {	//the site occupied by the seed randomly chosen from the local population
						int subpop = i / PopulationSizeForSeed;
						int start_ids = subpop * PopulationSizeForSeed - 1;
						int end_ids = start_ids + PopulationSizeForSeed;
						double SeedThreshold = 0.0;
						if(start_ids == -1){
							SeedThreshold = SeedCounter[end_ids] * runif(gen);
						} else {
							SeedThreshold = (SeedCounter[end_ids] - SeedCounter[start_ids]) * runif(gen) + SeedCounter[start_ids];
						}
						j = lower_bound(SeedCounter.begin() + start_ids, SeedCounter.begin() + end_ids, SeedThreshold) - SeedCounter.begin();
					} 
				} else {
					//This site becomes a vacant site
					N_Species[i] = Num_species-1;
					N_Harassment[i] = Minimum;
					continue;
				}
				
				// the seed of the individual "j" gets the site
				int Reproducing_species = Species[j];
				N_Species[i] = Reproducing_species;
				if (rbinom(gen)){
					//cout << i << " inherit from mother " << j <<endl;
					N_Harassment[i] = Harassment[j];
				} else {
					//cout << i << " inherit from father, the mother " << j;
					int subpop = j / PopulationSizeForPollen;
					//double Threshold = runif(gen) * (THarassment[subpop][Reproducing_species] - Harassment[j]);		//self incompatible
					double Threshold = runif(gen) * THarassment[subpop][Reproducing_species];
					int start_idp = subpop * PopulationSizeForPollen;
					int end_idp = start_idp + PopulationSizeForPollen;
					double HarassmentCounter = 0.0;

					for (int k = start_idp; k < end_idp; ++k){
						//if(k != j){		//self incompatible ver
						if (Reproducing_species == Species[k]){
							HarassmentCounter += Harassment[k];
							if (Threshold < HarassmentCounter){
								N_Harassment[i] = Harassment[k];
								//cout << " father " << k << endl;
								break;
							}
						}
						//}					//self incompatible ver
					}
				}
				if (MutationRate > runif(gen)){
					N_Harassment[i] += mutant(gen);	//mutation occurs
					if(N_Harassment[i] < 0.0){
						N_Harassment[i] = Minimum;
					}
				}
			}else{ //The individual survives to next generation
				N_Species[i] = Species[i];
				N_Harassment[i] = Harassment[i];
				//cout << i << " survive";
			}
			//cout << endl;
		}
		//cout << "--------------------------------Observe----------------------------------" << endl;
		for (i = 0; i < WholePopulationSize; ++i){
			Species[i] = N_Species[i];
			Harassment[i] = N_Harassment[i];
			N_Species[i] = -1;
			N_Harassment[i] = -1.0;
			Harassment_sp[t][Species[i]] += Harassment[i];
			++Number_sp[t][Species[i]];
		}
	}//end of time steps

	//cout << "----------------------------------Output1---------------------------------" << endl;
	//cout << tempDay[6] << tempDay[7] << tempDay[0] << tempDay[1] <<tempDay[3] << tempDay[4] << endl;
	ostringstream Filepath1;
	Filepath1 << time_str << "SP" << Num_species << "FPS" << PopulationSizeForSeed << "MPS" << PopulationSizeForPollen << "DR" << DispersalRate << "Rep" << argv[1] << ".csv";
	ofstream ofs1(Filepath1.str());

	for (int x = 0; x < TimeSteps; ++x){
		for (i = 0; i < Num_species; ++i){
			ofs1 << Number_sp[x][i] << ",";
		}
		ofs1 << ",";
		for (i = 0; i < Num_species; ++i){
			ofs1 << Harassment_sp[x][i] << ",";
		}
		ofs1 << ",";
		for (i = 0; i < Num_species; ++i){
			ofs1 << RealizedProductivity_sp[x][i] << ",";	//observation is different timing from that for the other two values
		}
		ofs1 << endl;
	}
	ofs1 << endl;
	return 0;
}